macroScript IMVU_Toolkit
	category:"IMVU"
	toolTip:""
(
	global IMVUFloater
	try (closeRolloutFloater IMVUFloater) catch()
	
	localVersion = "0.1.5"

	XScaleFactor = 1.0
	YScaleFactor = 1.0
	ZScaleFactor = 1.0
	windowWidth = 300
	windowHeight = 700


	rgx = dotNetClass "System.Text.RegularExpressions.RegEx"
	isUpdateAvailable = False
	fn CheckUpdate =
	(
		onlineVersion
		url = "https://create.imvu.com/api/studio/3dsmax"
		
		if servicePointManager == undefined do
			servicePointManager = dotNetClass "System.Net.ServicePointManager"
			servicePointManager.Expect100Continue = true
			servicePointManager.SecurityProtocol = (dotNetClass "System.Net.SecurityProtocolType").Tls12
		if httpWebRequest == undefined do
			httpWebRequest = (dotNetClass "System.Net.HttpWebRequest").Create(url)
			httpWebRequest.Method = "GET"
			httpWebRequest.Timeout = 1000
			httpWebRequest.UserAgent = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_11_5) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/50.0.2661.102 Safari/537.36"
			httpWebRequest.proxy = (dotNetClass "System.Net.WebProxy").GetDefaultProxy()
		response = httpWebRequest.GetResponse()
		responseStream = response.GetResponseStream()
		stream = dotNetObject "System.IO.StreamReader" responseStream
		body = stream.ReadToEnd()
		pattern = "[^version\"]([0-9.]+)"
		match = rgx.Match body pattern
		if body != undefined do
			onlineVersion = match.value
		response.Close();
		if onlineVersion == undefined then
			print("ERROR: Version not found!")
		else
			if onlineVersion != localVersion do
				isUpdateAvailable = True
	)
	CheckUpdate()

	


	megaScene = ((pathConfig.removePathLeaf (getSourceFileName())) as string) + "\IMVUMegaScene.max"
	AttachmentScene = #("Attachment_Scene")
	ClothingScene = #("Clothing_Scene")
	AnimationScene = #("Animation_Scene")

	updateText = ""
	isAutoOn = False
		
		
	if isUpdateAvailable do
		updateText = "UPDATE AVAILABLE!"
	if not isUpdateAvailable do
		updateText = "Alpha " + localVersion

	fn bone_filt obj = classof obj == BoneGeometry
	
	fn setScales x y z=
	(
		XScaleFactor = x
		YScaleFactor = y
		ZScaleFactor = z
	)

	fn resizeRoom xyz createRoomButton XSlider YSlider ZSlider=
	(
		isRoom = False
		meshRef = ""
		
		newX = XSlider.value * 100
		newY = YSlider.value * 100
		newZ = ZSlider.value * 100
		
		for OBJ in Geometry do
		(
			if OBJ.name == "room_MESH" do
			(
				isRoom = True
				meshRef = OBJ
				createRoomButton.enabled = false
			)
		)
		
		if isRoom do
		(
			meshRef.scale = [newX / XScaleFactor, newY / YScaleFactor, newZ / ZScaleFactor]
		)
	)

	fn AddRoomSeat seatType ifCall_H ifCall_CP=
	(
		numSeats = 1
		seatNum = formattedPrint numSeats format:"02d"
		
		seatList = #()
		destination = #()
		Objs = for o in objects collect o
		for o in Objs do
		(
			parentSTR = o.parent as string
			trim = trimleft parentSTR "$Dummy:"
			parentSTR = trimright ((filterstring trim "@")[1])
			
			if parentSTR == "_AvatarNodes" do
			(
				append seatList o.name
			)
		)
		
		for obj in (sort seatList) do
		(
			if(matchPattern obj pattern:"*seat*") do
			(
				numSeats += 1
			)
			seat_str = numSeats
			seatNum = formattedPrint seat_str format:"02d"
		)
		
		dummySeat = Dummy name:("seat" + seatNum as string + "." + seatType as string) pos:[0,0,0] scale:[20,20,20]
		FOR obj in objects DO
			if obj.name == "_AvatarNodes" do
				dummySeat.parent = obj
				append destination obj
			
		if ifCall_H then
		(
			HSeat = Dummy name:("Handle" + seatNum as string) pos:[0,0,500] scale:[20,20,20]
			FOR obj in objects WHERE isGroupHead obj DO
				if obj.name == "_AvatarNodes" do
					append destination obj
					HSeat.parent = dummySeat
		)
		else
			break
			
		if ifCall_CP then
		(
			CSeat = Dummy name:("Catcher" + seatNum as string + "." + seatType) pos:[0,-300,0] scale:[15,15,15]
			CSeat.parent = dummySeat
			PSeat = Dummy name:("Pitcher" + seatNum as string + "." + seatType) pos:[0,-800,0] scale:[15,15,15]
			PitchRot = eulerangles 0 0 180
			rotate PSeat PitchRot
			PSeat.parent = CSeat
		)
		else
			break
	)

	fn AddFurnitureSeat seatType ifCall_H ifCall_CP=
	(
		
		numSeats = 1
		seatNum = formattedPrint numSeats format:"02d"
		
		seatList = #()
		destination = #()
		Objs = for o in objects collect o
		for o in Objs do
		(
			parentSTR = o.parent as string
			trim = trimleft parentSTR "$Dummy:"
			parentSTR = trimright ((filterstring trim "@")[1])
			
			if parentSTR == "_Nodes" do
			(
				append seatList o.name
			)
		)
		
		for obj in (sort seatList) do
		(
			if(matchPattern obj pattern:"*seat*") do
			(
				numSeats += 1
			)
			seat_str = numSeats
			seatNum = formattedPrint seat_str format:"02d"
		)
		
		dummySeat = Dummy name:("seat" + seatNum as string + "." + seatType as string) pos:[0,0,0] scale:[20,20,20]
		FOR obj in objects DO
			if obj.name == "_Nodes" do
				dummySeat.parent = obj
				

			
		if ifCall_H then
		(
			HSeat = Dummy name:("Handle" + seatNum as string) pos:[0,0,500] scale:[20,20,20]
			HSeat.parent = dummySeat
		)
		else
			break
			
		if ifCall_CP then
		(
			CSeat = Dummy name:("Catcher" + seatNum as string + "." + seatType) pos:[0,-300,0] scale:[15,15,15]
			CSeat.parent = dummySeat
			PSeat = Dummy name:("Pitcher" + seatNum as string + "." + seatType) pos:[0,-800,0] scale:[15,15,15]
			PitchRot = eulerangles 0 0 180
			rotate PSeat PitchRot
			PSeat.parent = CSeat
		)
		else
			break
	)

	fn SetTimeScale animLength=
	(
		animationRange = interval 0 animLength.value
	)

	fn GW_displayObjectNames =
	(
		gw.setTransform (matrix3 1)
		for o in objects where not o.isHiddenInVpt do
		(
			gw.text o.pos o.name color:yellow
			gw.enlargeUpdateRect #whole
		)
		
	)

	fn dupFurn dir dist =
	(
		objs = #()
		for each in selection do
		(
			append objs each.name
		)
		x_amount = 0
		y_amount = 0
		z_amount = 0

		if dir == "N" do
			y_amount = dist
		if dir == "S" do
			y_amount = -dist
		if dir == "E" do
			x_amount = dist
		if dir == "W" do
			x_amount = -dist

		for sel in $ do
		(
			s = sel.name
			if matchPattern s pattern:"furniture.Ceiling*" do
			(
				x_amount = -x_amount
			)
		)
		
		isSeat = False
		isFurn = False

		if matchPattern objs[1] pattern:"furniture*"ignoreCase:false do
		(
			isFurn = True
		)
		if matchPattern objs[1] pattern:"seat*"ignoreCase:false do
		(
			isSeat = True
		)

		if isSeat do
		(
			seat_str = ""
			numSeats = 1
			seatNum = formattedPrint numSeats format:"02d"
			
			seatList = #()
			destination = #()
			listObjs = for o in objects collect o
			for o in listObjs do
			(
				parentSTR = o.parent as string
				trim = trimleft parentSTR "$Dummy:"
				parentSTR = trimright ((filterstring trim "@")[1])
				
				if parentSTR == "Seating" do
				(
					append seatList o.name
				)
			)
			
			for obj in (sort seatList) do
			(
				if(matchPattern obj pattern:"*seat*") do
				(
					numSeats += 1
				)
				seat_str = numSeats-1
			)
			maxOps.cloneNodes selection cloneType:#copy newNodes:&nnl
			select nnl
			in coordsys #local nnl.pos += [x_amount*100, y_amount*100, z_amount*100]

			newSeat = nnl
			seat_int = seat_str as integer
			for each in newSeat do
			(
				seat_int += 1
				seatNum = formattedPrint seat_int format:"02d"
				each.name = ("seat" + (seatNum as string) + "." + (filterstring objs[1] ".")[2])
			)

			
		)
		if isFurn do
		(
			if(matchPattern objs[1] pattern:"*Floor*") do
			(
				furnType = "Floor."

				numFloors = 1
				floorNum = formattedPrint numFloors format:"02d"
				
				floorList = #()
				destination = #()
				newObjs = for o in objects collect o
				for o in newObjs do
				(
					parentSTR = o.parent as string
					trim = trimleft parentSTR "$Dummy:"
					parentSTR = trimright ((filterstring trim "@")[1])
					
					if parentSTR == "_FurnitureNodes" do
					(
						append floorList o.name
					)
				)
				
				for obj in (sort floorList) do
				(
					if(matchPattern obj pattern:"*Floor*") do
					(
						numFloors += 1
					)
					seat_str = numFloors - 1
				)

				maxOps.cloneNodes selection cloneType:#copy newNodes:&nnl
				select nnl
				-- move nnl [x_amount*100, y_amount*100, z_amount*100]
				in coordsys local nnl.pos += [x_amount*100, y_amount*100, z_amount*100]

				newFurn = nnl
				seat_int = seat_str as integer
				for each in newFurn do
				(
					seat_int += 1
					floorNum = formattedPrint seat_int format:"02d"
					each.name = (substring each.name 1 16) + floorNum
				)
			)

			if(matchPattern objs[1] pattern:"*Wall*") do
			(
				furnType = "Wall."

				numWalls = 1
				wallNum = formattedPrint numWalls format:"02d"
				
				wallList = #()
				destination = #()
				newObjs = for o in objects collect o
				for o in newObjs do
				(
					parentSTR = o.parent as string
					trim = trimleft parentSTR "$Dummy:"
					parentSTR = trimright ((filterstring trim "@")[1])
					
					if parentSTR == "_FurnitureNodes" do
					(
						append wallList o.name
					)
				)
				
				for obj in (sort wallList) do
				(
					if(matchPattern obj pattern:"*Wall*") do
					(
						numWalls += 1
					)
					seat_str = numWalls - 1
				)

				maxOps.cloneNodes selection cloneType:#copy newNodes:&nnl
				select nnl
				-- move nnl [x_amount*100, z_amount*100, y_amount*100]
				in coordsys local nnl.pos += [x_amount*100, y_amount*100, z_amount*100]

				newFurn = nnl
				seat_int = seat_str as integer
				for each in newFurn do
				(
					seat_int += 1
					floorNum = formattedPrint seat_int format:"02d"
					each.name = (substring each.name 1 15) + floorNum
				)
			)

			if(matchPattern objs[1] pattern:"*Ceiling*") do
			(
				furnType = "Ceiling_"

				numCeilings = 1
				ceilingNum = formattedPrint numCeilings format:"02d"
				
				ceilingList = #()
				destination = #()
				newObjs = for o in objects collect o
				for o in newObjs do
				(
					parentSTR = o.parent as string
					trim = trimleft parentSTR "$Dummy:"
					parentSTR = trimright ((filterstring trim "@")[1])
					
					if parentSTR == "_FurnitureNodes" do
					(
						append ceilingList o.name
					)
				)
				
				for obj in (sort ceilingList) do
				(
					if(matchPattern obj pattern:"*Ceiling*") do
					(
						numCeilings += 1
					)
					seat_str = numCeilings - 1
				)

				maxOps.cloneNodes selection cloneType:#copy newNodes:&nnl
				select nnl
				-- move nnl [x_amount*100, y_amount*100, z_amount*100]
				in coordsys local nnl.pos += [x_amount*100, y_amount*100, z_amount*100]

				newFurn = nnl
				seat_int = seat_str as integer
				for each in newFurn do
				(
					seat_int += 1
					ceilingNum = formattedPrint seat_int format:"02d"
					each.name = (substring each.name 1 18) + ceilingNum
				)
			)
		)
	)

	fn AddNode nodeType =
	(
		obj = selection
		isSeat = True

		if isSeat do
		(
			seatSelection = selection
			for i=1 to selection.count do
			(
				seatNum = (substring ((filterstring obj[i].name ".")[1]) 5 -1)
				seatType = (filterstring obj[i].name ".")[2]
				
				if nodeType == "Add Catch" do
				(
					isCatch = False
					for each in obj[i] do
					(
						if matchPattern each.name pattern:"*Catcher*" do
						(
							isCatch = True
						)
					)
					if not isCatch do
					(
						CSeat = Dummy name:("Catcher" + seatNum as string + "." + seatType) pos: (obj[i].pos + [0,-300,0]) boxsize:[150,150,150]
						CSeat.parent = obj[i]
						PSeat = Dummy name:("Pitcher" + seatNum as string + "." + seatType) pos: (obj[i].pos + [0,-800,0]) boxsize:[150,150,150]
						PitchRot = eulerangles 0 0 180
						rotate PSeat PitchRot
						PSeat.parent = CSeat
					)
				)

				if nodeType == "Del Catch" do
				(
					isCatch = False
					for each in obj[i] do
					(
						if matchPattern each.name pattern:"*Catcher*" do
						(
							namesArray = #()
							for s in selection do append namesArray s
							select each
							selectMore each.children
							actionMan.executeAction 0 "40020"
							for oldSele in namesArray do selectMore oldSele
							
						)
					)
				)

				if nodeType == "Add Handle" do
				(
					isHandle = False
					for each in obj[i] do
					(
						if matchPattern each.name pattern:"*Handle*" do
						(
							isHandle = True
						)
					)
					if not isHandle do
					(
						Handle = Dummy name:("Handle" + seatNum as string) pos: (obj[i].pos + [0,0,500]) boxsize:[150,150,150]
						Handle.parent = obj[i]
					)
				)
				if nodeType == "Del Handle" do
				(
					isHandle = False
					for each in obj[i] do
					(
						if matchPattern each.name pattern:"*Handle*" do
						(
							namesArray = #()
							for s in selection do append namesArray s
							select each
							selectMore each.children
							actionMan.executeAction 0 "40020"
							for oldSele in namesArray do selectMore oldSele
							
						)
					)
				)
			)
		)
		
		redrawViews()
	)

	rollout VisualsRollout "Visuals" rolledup:true 
	(
		
		group ""
		(
			button showNames "Show Names" width:110 toolTip: "Show Object Names"
			button hideNames "Hide Names" width:110 toolTip: "Hide Object Names"
			button spacer01 "" height:1 width:0
		)
		
		
		on showNames pressed do
		(
			unregisterRedrawViewsCallback GW_displayObjectNames
			registerRedrawViewsCallback GW_displayObjectNames
		)
		
		on hideNames pressed do
		(
			unregisterRedrawViewsCallback GW_displayObjectNames
			
		)
	)

	rollout RoomRollout "Room Tool" rolledup:true 
	(
		group ""
		(
			spinner		XSlider 	"X" 	range:[0, 100, 40] 	width:65 	align:#left 	enabled:False	across:3 
			spinner 	YSlider 	"Y" 	range:[0, 100, 40] 	width:65 	align:#center	enabled:False
			spinner 	ZSlider 	"Z" 	range:[0, 100, 40] 	width:65 	align:#right	enabled:False

			button 		createRoomButton 	"Create Room" 	width:220 	toolTip: "Create a room based on the above dimensions"	enabled:True
			button 		spacer01 			"" 				width:0		height:1
		)
		
		group ""
		(
			radiobuttons 	furnitureType 		labels:#("Floor      ","Wall        ","Ceiling") 	enabled:False
			radiobuttons 	furnitureDir 		labels:#("North   ","South   ","East   ","West") 	enabled:False
			button 			addFurnitureButton 	"+  Furniture" 	width:220							enabled:False	toolTip:"Create a piece of furniture based on the above selection"
		)
		
		group ""
		(
			button		roomN		"Up"		width:50 	toolTip:"Duplicate furniture up" 		enabled:False
			button		roomW		"Left"		width:50 	toolTip:"Duplicate furniture left"		enabled:False 	align:#right	across:3
			spinner 	furnDist	""			width:50 	toolTip:"How much to duplicate by" 		enabled:False 	range:[0, 100, 3] align:#center
			button		roomE		"Right"		width:50 	toolTip:"Duplicate furniture right" 	enabled:False	align:#left
			button		roomS		"Down"		width:50 	toolTip:"Duplicate furniture down" 		enabled:False
		)

		group ""
		(
			button 		addIdleButton 		"+  Idle" 		across:1 	width:220 	align:#center	toolTip:"Create an idle seat node" 		enabled:False
			button 		addStandingButton 	"+  Standing" 	across:1 	width:220 	align:#center	toolTip:"Create a standing seat node" 	enabled:False
			button 		addSittingButton 	"+  Sitting" 	across:1 	width:220 	align:#center	toolTip:"Create a sitting seat node" 	enabled:False
			
		)
		
		group ""
		(
			edittext 	customText 			"" 				width:220 	align:#center 	tooltip:"Name the custom seat node"		enabled:False
			button 		addCustomButton 	"+  Custom" 	width:220 	align:#center 	toolTip:"Create a custom seat node" 	enabled:False
		)
		
		group ""
		(
			button 	addCatcher 		"+  Catcher" 	width:110 	toolTip:"Add a catcher node" 	enabled:False 	align:#right	across:2 
			button 	addHandle 		"+  Handle" 	width:110 	toolTip:"Remove a handle node" 	enabled:False	align:#left
			button 	removeCatcher 	"-  Catcher" 	width:110 	toolTip:"Add a catcher node" 	enabled:False	align:#right 	across:2 
			button 	removeHandle 	"-  Handle" 	width:110 	toolTip:"Remove a handle node" 	enabled:False	align:#left
		)
		
		group ""
		(
			button 	createRoomST 	"Create See Through Node" 	width:220 	align:#center 	enabled:False
			button 	linkRoomST 		"Link To SeeThrough" 		width:220 	align:#center 	enabled:False
			button 	unlinkRoomST 	"Unlink From SeeThrough" 	width:220 	align:#center 	enabled:False
		)
		
		
		on furnitureType changed state do
		(
			if furnitureType.state == 2 then
			(
				furnitureDir.enabled = True
			)
			else
			(
				furnitureDir.enabled = False
			)
		)
		
		on XSlider changed val do
		(
			resizeRoom "X" createRoomButton Xslider YSlider ZSlider
		)
		on YSlider changed val do
		(
			resizeRoom "Y" createRoomButton Xslider YSlider ZSlider
		)
		on ZSlider changed val do
		(
			resizeRoom "Z" createRoomButton Xslider YSlider ZSlider
		)
		

		on createRoomButton pressed do --Create Room
		(
			createRoomButton.enabled = False
			buttonsGroup = #(XSlider, YSlider, ZSlider, furnitureType, addFurnitureButton, 
				addIdleButton, addStandingButton, addSittingButton,
				customText, addCustomButton, createRoomST, linkRoomST, unlinkRoomST,
				addCatcher, addHandle, removeCatcher, removeHandle,
				roomN, roomW, roomE, rooms, furnDist)
				
			for b in buttonsGroup do b.enabled = True
			createRoomButton.enabled = False
			for OBJ in Geometry do
			(
				if OBJ.name == "Room_Ref" do
				(
					delete OBJ
				)
			)
			
			newX = XSlider.value*100
			newY = YSlider.value*100
			newZ = ZSlider.value*100
			
			setScales newX newY newZ
			
			skeletonRoom = Dummy name:"skeleton.Room" pos:[0,0,0] boxsize:[400,400,400]
			RoomNodesGRP = group skeletonRoom name:"ROOM_EXPORT"
			camRoot = Dummy name:"camera.01.01.root" pos:[1000,-2800,500] boxsize:[300,300,300]
			rotate camRoot (angleaxis 90 [1,0,0])
			camTarget = Dummy name:"camera.01.01.Target" pos:[1000,1000,600] boxsize:[300,300,300]
			omni = Omnilight name:"Omni01" rgb:(color 255 255 255) shadowColor:(color 0 0 0) multiplier:1 contrast:0 softenDiffuseEdge:0 nearAttenStart:0 nearAttenEnd:40 farAttenStart:80 farAttenEnd:200 decayRadius:40 atmosOpacity:100 atmosColorAmt:100 shadowMultiplier:1 pos:[500,0,2500]
			avatarNodes = Dummy name:"_AvatarNodes" pos:[0,0,0] boxsize:[0,0,0]
			furnitureNodes = Dummy name:"_FurnitureNodes" pos:[0,0,0] boxsize:[0,0,0]
			lightingNodes = Dummy name:"_LightingNodes" pos:[0,0,0] boxsize:[0,0,0]

			max modify mode
			roomMesh = box width:newX length:newY height:newZ
			roomMesh.name = "Room_Ref"
			convertTo roomMesh PolyMeshObject
			select roomMesh
			subobjectLevel = 4
			roomMesh.EditablePoly.SetSelection #Face #{1..6}
			roomMesh.EditablePoly.flipNormals 1
			roomMesh.backfacecull = on
			subobjectLevel = 5
			roomMesh.EditablePoly.SetSelection #Face #{}
			greyMat = standardMaterial name:"M_Room" diffuse:(color 204 204 204)
			roomMesh.material = greyMat
			
			roomMesh.parent = skeletonRoom
			omni.parent = lightingNodes
			avatarNodes.parent = skeletonRoom
			furnitureNodes.parent = skeletonRoom
			lightingNodes.parent = skeletonRoom

			select camRoot
			theCtrl = LookAt_Constraint relative:true
			theCtrl.appendTarget camTarget 100
			selection.rotation.controller = theCtrl
			
			camRoot.parent = skeletonRoom
			camTarget.parent = skeletonRoom
			
			scaleCube = box name:"ScaleCube_Ref" width:313 length:313 height:313 pos:[-1500, 0, 0]
			scaleCube.parent = skeletonRoom

			seats = #()
			
			for i = 1 to 6 do
			(
				dummySeat = Dummy name:("seat0" + (i as string) + ".Standing") pos:[((i-1)*300),0,0] boxsize:[200,200,200]
				dummySeat.parent = AvatarNodes
			)
			select #($ROOM_EXPORT)
			actionMan.executeAction 0 "63561"  -- Groups: Group Open Recursively
			
		)
		
		on addFurnitureButton pressed do --Add Furniture
		(
			
			if furnitureType.state == 1 do --Floor
			(
				numFloors = 1
				floorNum = formattedPrint numFloors format:"02d"
				
				floorList = #()
				destination = #()
				Objs = for o in objects collect o
				for o in Objs do
				(
					parentSTR = o.parent as string
					trim = trimleft parentSTR "$Dummy:"
					parentSTR = trimright ((filterstring trim "@")[1])
					
					if parentSTR == "_FurnitureNodes" do
					(
						append floorList o.name
					)
				)
				
				for obj in (sort floorList) do
				(
					if(matchPattern obj pattern:"*Floor*") do
					(
						numFloors += 1
					)
					seat_str = numFloors
					floorNum = formattedPrint seat_str format:"02d"
				)
				
				dummyFurniture = Dummy name:("furniture.Floor." + floorNum as string) pos:[0,0,0] boxsize:[200,200,200]
				dummyFurniture.rotation.controller = Euler_XYZ ()
				for obj in objects do
					if obj.name == "_FurnitureNodes" do
						dummyFurniture.parent = obj
				
			)
			if furnitureType.state == 2 do --Wall
			(
				numWalls = 1
				wallNum = formattedPrint numWalls format:"02d"
				
				wallList = #()
				destination = #()
				Objs = for o in objects collect o
				for o in Objs do
				(
					parentSTR = o.parent as string
					trim = trimleft parentSTR "$Dummy:"
					parentSTR = trimright ((filterstring trim "@")[1])
					
					if parentSTR == "_FurnitureNodes" do
					(
						append wallList o.name
					)
				)
				
				for obj in (sort wallList) do
				(
					if(matchPattern obj pattern:"*Wall*") do
					(
						numWalls += 1
					)
					seat_str = numWalls
					wallNum = formattedPrint seat_str format:"02d"
				)
				
				posX = XSlider.value*50
				posY = YSlider.value*50
				posZ = ZSlider.value*50
				
				rotZ = 0

				if furnitureDir.state == 1 or furnitureDir.state == 2 then
				(
					if furnitureDir.state == 1 then
					(
						posX = 0
						rotZ = -180
					)
					else
					(
						posX = 0
						posY *= -1
						rotZ = 0
					)
				)
				else
				(
					if furnitureDir.state == 3 then
					(
						posY = 0
						rotZ = 90
					)
					else
					(
						posX *= -1
						posY = 0
						rotZ = -90
					)
				)
				

				dummyFurniture = Dummy name:("furniture.Wall." + wallNum as string) pos:[posX,posY,posZ] boxsize:[200,200,200]
				dummyFurniture.rotation.controller = Euler_XYZ ()
				wallRot = eulerangles -90 180 rotZ
				rotate dummyFurniture wallRot
				
				for obj in objects do
					if obj.name == "_FurnitureNodes" do
						dummyFurniture.parent = obj
				
			)
			if furnitureType.state == 3 do --Ceiling
			(
				numCeils = 1
				ceilingNum = formattedPrint numCeils format:"02d"
				
				ceilingList = #()
				destination = #()
				Objs = for o in objects collect o
				for o in Objs do
				(
					parentSTR = o.parent as string
					trim = trimleft parentSTR "$Dummy:"
					parentSTR = trimright ((filterstring trim "@")[1])
					
					if parentSTR == "_FurnitureNodes" do
					(
						append ceilingList o.name
					)
				)
				
				for obj in (sort ceilingList) do
				(
					if(matchPattern obj pattern:"*Ceiling*") do
					(
						numCeils += 1
					)
					seat_str = numCeils
					ceilingNum = formattedPrint seat_str format:"02d"
				)
				
				dummyFurniture = Dummy name:("furniture.Ceiling." + ceilingNum as string) pos:[0,0,ZSlider.value*100] boxsize:[200,200,200]
				dummyFurniture.rotation.controller = Euler_XYZ ()
				ceilRot = eulerangles 0 180 0
				rotate dummyFurniture ceilRot
				
				for obj in objects do
					if obj.name == "_FurnitureNodes" do
						dummyFurniture.parent = obj
				
			)
		)
		
		on roomN pressed do --Duplicate Furniture North
		(
			canDup = True
			for each in selection do
			(
				if not matchpattern each.name pattern:"*furniture*" do
				(
					canDup = False
					messageBox "Only furniture nodes can be duplicated"
				)
			)
			if canDup do
			(
				dupFurn "N" furnDist.value
			)
		)

		on roomS pressed do --Duplicate Furniture South
		(
			dupFurn "S" furnDist.value
		)

		on roomE pressed do --Duplicate Furniture East
		(
			dupFurn "E" furnDist.value
		)

		on roomW pressed do --Duplicate Furniture West
		(
			dupFurn "W" furnDist.value
		)

		on addSittingButton pressed do --Add Sitting
		(
			AddRoomSeat "Sitting" true true
		)
		
		on addStandingButton pressed do --Add Standing
		(
			AddRoomSeat "Standing" true true
		)
		
		on addIdleButton pressed do --Add Idle
		(
			AddRoomSeat "Idle" true true
		)
		
		on addCustomButton pressed do --Add Custom
		(
			AddRoomSeat customText.text true true
		)

		on addCatcher pressed do
		(
			AddNode "Add Catch"
		)

		on addHandle pressed do
		(
			AddNode "Add Handle"
		)

		on removeCatcher pressed do
		(
			AddNode "Del Catch"
		)

		on removeHandle pressed do
		(
			AddNode "Del Handle"
		)
		
		on createRoomST pressed do --Create SeeThrough
		(
			numSeeThrough = 1
			
			Objs = for o in objects collect o
			for o in Objs do
			(
				if(matchPattern o.name pattern:"*SeeThrough*") do
				(
					numSeeThrough += 1
				)
				
			)
			
			seeThroughNum = formattedPrint numSeeThrough format:"02d"
			SeeThroughNode = Cone radius1:100 radius2:-100 height:200
			SeeThroughNode.name = ("SeeThrough" + seeThroughNum as string)
			skeletonRoom = $'skeleton_Room'
			SeeThroughNode.parent = skeletonRoom
			
			wallRot = eulerangles 90 0 0
			rotate SeeThroughNode wallRot
			
			invisMat = standardMaterial name:"M_SeeThrough" opacity:50
			SeeThroughNode.material = invisMat
			
			select SeeThroughNode
		)
		
		on linkRoomST pressed do --Link SeeThrough
		(
			verts = #()
			meshes = #()

			isArmature = False
			isMesh = False
			numArmature = 0
			seeThroughNum = 0
			alreadyST = false

			Objs = for o in selection collect o
			if Objs.count == 0 then
			(
				messageBox "Select a SeeThrough cone and an object to bind to"
			)
			else
			(
				for o in Objs do
				(
					alreadyST = false
					if(matchPattern o.name pattern:"*SeeThrough*") then
					(
						isArmature = True
						seeThroughNum = trimLeft o.name "SeeThrough"
						numArmature += 1
					)
					else
					(
						for each in o.modifiers do
						(
							if (matchPattern each.name pattern:"STSkin") then
							(
								alreadyST = true
							)
						)
						if not alreadyST do
						(
							append meshes o
							isMesh = True
						)
					)
				)
				if numArmature > 1 do messageBox "Please select only one SeeThrough node!"
				if not isArmature do messageBox "Please select an armature to attach!"
				if isArmature and isMesh do
				(
					objToLink = ("$SeeThrough"+ (seeThroughNum as string))
					print objToLink
					for eachMesh in meshes do
					(
						suspendEditing which:#modify
						tMod = Skin()
						addModifier eachMesh tMod
						tMod.name = "STSkin"
						resumeEditing which:#modify
						modPanel.setCurrentObject eachMesh.modifiers[#STSkin]
						skinOps.addBone eachMesh.modifiers[#STSkin] $SeeThrough01 1
					)
				)
			)
		)

		on unlinkRoomST pressed do --Unlink SeeThrough
		(
			Objs = for o in selection collect o
			if Objs.count == 0 then
			(
				messageBox "Please select an object!"
			)
			else
			(
				for eachMesh in Objs do
				(
					suspendEditing which:#modify
					deleteModifier eachmesh (eachmesh.modifiers[#STSkin])
					
					resumeEditing which:#modify
				)
			)


		)
	)

	rollout FurnitureRollout "Furniture Tool" rolledup:true 
	(
		
		group ""
		(
			button createFurnitureButton "Create Furniture" width:220 toolTip: "Create a furniture outline"
			button spacer01 "" height:1 width:0
		)
		
		group ""
		(
			button F_addIdleButton "+  Idle" across:1 width:220 align:#center toolTip: "Create an idle seat node" enabled: False
			
			button F_addStandingButton "+  Standing" across:1 width:220 align:#center toolTip: "Create an idle seat node" enabled: False
			
			button F_addSittingButton "+  Sitting" across:1 width:220 align:#center toolTip: "Create an idle seat node" enabled: False
			
		)
		
		group ""
		(
			edittext F_customText "" width:220 align:#center enabled: False
			button F_addCustomButton "+  Custom" across:1 width:220 align:#center toolTip: "Create an idle seat node" enabled: False
		)
		
		group ""
		(
			button 	F_addCatcher 	"+  Catcher" 	width:110 	across:2	align:#right	toolTip:"Add a catcher node" 		enabled:False
			button 	F_addHandle 	"+  Handle" 	width:100 				align:#left	toolTip:"Remove a handle node" 		enabled:False
			button 	F_removeCatcher "-  Catcher"	width:110 	across:2  	align:#right	toolTip:"Add a catcher node" 		enabled:False
			button 	F_removeHandle 	"-  Handle" 	width:100 				align:#left	toolTip:"Remove a handle node" 		enabled:False
		)
		
		group ""
		(
			button 	F_createRoomST 	"Create See Through Node" 	width:220 	align:#center 	enabled:False
			button 	F_linkRoomST 	"Link To SeeThrough" 		width:220 	align:#center 	enabled:False
			button 	F_unlinkRoomST 	"Unlink From SeeThrough" 	width:220 	align:#center 	enabled:False
		)
		
		on createFurnitureButton pressed do --Create Furniture
		(
			createFurnitureButton.enabled = False
			buttonsGroup = #(F_addIdleButton, F_addStandingButton,
				F_addSittingButton, F_customText, F_addCustomButton,
				F_addCatcher, F_addHandle, F_removeCatcher, F_removeHandle,
				F_createRoomST, F_linkRoomST, F_unlinkRoomST)
				
			for b in buttonsGroup do b.enabled = True
			createFurnitureButton.enabled = False
			
			scaleCube = box name:"scaleCube_Ref" width:600 length:313 height:313 pos:[0, 0, 0]
			dummyMesh = Dummy name:"_Meshes" pos:[0,-0,0] scale:[30,30,30]
			scaleCube.parent = dummyMesh
			
			dummyNodes = Dummy name:"_Nodes" pos:[0,-0,0] scale:[20,20,20]
			dummyRoot = Dummy name:"Attatchment.Root" pos:[0,-0,0] scale:[15,15,15]
			dummyMesh.parent = dummyRoot
			dummyNodes.parent = dummyRoot
			
			FurnitureNodesGRP = group dummyRoot name:"FURNITURE_EXPORT"
			
			select #($FURNITURE_EXPORT)
			actionMan.executeAction 0 "63561"  -- Groups: Group Open Recursively
		)
		
		on F_addSittingButton pressed do --Add Sitting
		(
			AddFurnitureSeat "Sitting" true true
		)
		
		on F_addStandingButton pressed do --Add Standing
		(
			AddFurnitureSeat "Standing" true true
		)
		
		on F_addIdleButton pressed do --Add Idle
		(
			AddFurnitureSeat "Idle" true true
		)
		
		on F_addCustomButton pressed do --Add Custom
		(
			AddFurnitureSeat F_customText.text true true
		)
		
		on F_addCatcher pressed do
		(
			AddNode "Add Catch"
		)

		on F_addHandle pressed do
		(
			AddNode "Add Handle"
		)

		on F_removeCatcher pressed do
		(
			AddNode "Del Catch"
		)

		on F_removeHandle pressed do
		(
			AddNode "Del Handle"
		)
		
		on F_createRoomST pressed do --Create SeeThrough
		(
			numSeeThrough = 1
			
			Objs = for o in objects collect o
			for o in Objs do
			(
				if(matchPattern o.name pattern:"*SeeThrough*") do
				(
					numSeeThrough += 1
				)
				
			)
			
			seeThroughNum = formattedPrint numSeeThrough format:"02d"
			SeeThroughNode = Cone radius1:100 radius2:-100 height:200
			SeeThroughNode.name = ("SeeThrough" + seeThroughNum as string)
			skeletonRoom = $'skeleton_Room'
			SeeThroughNode.parent = skeletonRoom
			
			invisMat = standardMaterial name:"M_SeeThrough" opacity:50
			SeeThroughNode.material = invisMat
			
			select SeeThroughNode
		)
		
		on F_linkRoomST pressed do --Link SeeThrough
		(
			verts = #()
			meshes = #()

			isArmature = False
			isMesh = False
			numArmature = 0
			seeThroughNum = 0
			alreadyST = false

			Objs = for o in selection collect o
			if Objs.count == 0 then
			(
				messageBox "Select a SeeThrough cone and an object to bind to"
			)
			else
			(
				for o in Objs do
				(
					alreadyST = false
					if(matchPattern o.name pattern:"*SeeThrough*") then
					(
						isArmature = True
						seeThroughNum = trimLeft o.name "SeeThrough"
						numArmature += 1
					)
					else
					(
						if (o.modifiers.count == 0) do
						(
							append meshes o
							isMesh = True
						)
						for each in o.modifiers do
						(
							if (matchPattern each.name pattern:"STSkin") then
							(
								alreadyST = true
							)
							if not alreadyST then
							(
								append meshes o
								isMesh = True
							)
						)
					)
				)
				if numArmature > 1 do messageBox "Please select only one SeeThrough node!"
				if isArmature and not isMesh do messageBox "Please select an object to attach!"
				if not isArmature do messageBox "Please select an armature to attach!"
				if isArmature and isMesh do
				(
					objToLink = ("$SeeThrough"+ (seeThroughNum as string))
					for eachMesh in meshes do
					(

						print(eachMesh.modifiers)
						suspendEditing which:#modify
						tMod = Skin()
						addModifier eachMesh tMod
						tMod.name = "STSkin"
						resumeEditing which:#modify
						modPanel.setCurrentObject eachMesh.modifiers[#STSkin]
						skinOps.addBone eachMesh.modifiers[#STSkin] $SeeThrough01 1
					)
				)
			)
		)
		
		on F_unlinkRoomST pressed do --Unlink SeeThrough
		(
			Objs = for o in selection collect o
			if Objs.count == 0 then
			(
				messageBox "Please select an object!"
			)
			else
			(
				for eachMesh in Objs do
				(
					suspendEditing which:#modify
					deleteModifier eachmesh (eachmesh.modifiers[#STSkin])
					
					resumeEditing which:#modify
				)
			)


		)
	)

	rollout AttachmentRollout "Attachment Tool" rolledup:true 
	(
		
		group ""
		(
			button appendAttachmentFile "Create Attachment" width:220 toolTip: "Create a new attachment product"
			button spacer01 "" height:1 width:0
		)

		group ""
		(
			radiobuttons 	avatarType 		labels:#("Male      ","Female        ","Hide") 	enabled:False

			label			boneLabel		"Bone:     "		width:80			across:2		enabled:False
			--dropdownlist	bone_dd 		"" 			width:110 	align:#left 	items:#("PelvisNode", 
			--	"lfHip", "lfThigh", "lfCalf", "lfFoot", "lfToes", 
			--	"rtHip", "rtThigh", "rtCalf", "rtFoot", "rtToes", "Spine01", 
			--	"Spine02", "Spine03", "Spine04", "Neck01", "Neck02", "Neck03", "Neck04", "Head", 
			--	"lfClavicle", "lfShoulder", "lfbicep", "lfElbow", "lfWrist", "lfHand", "lfmetaCarpal03", 
			--	"lfFingerMiddle01", "lfFingerMiddle02", "lfFingerMiddle03", "lfmetaCarpal01", 
			--	"lfThumb01d", "lfThumb02", "lfThumb03", "lfmetaCarpal05", "lfFingerPinky01", 
			--	"lfFingerPinky02", "lfFingerPinky03", "lfmetaCarpal02", "lfFingerIndex01", 
			--	"lfFingerIndex02", "lfFingerIndex03", "lfmetaCarpal04", "lfFingerRing01", "lfFingerRing02", 
			--	"lfFingerRing03", "rtClavicle", "rtShoulder", "rtBicep", "rtElbow", "rtWrist", "rtHand", 
			--	"rtmetaCarpal03", "rtFingerMiddle01", "rtFingerMiddle02", "rtFingerMiddle03", "rtmetaCarpal01", 
			--	"rtThumb01", "rtThumb02", "rtThumb03", "rtmetaCarpal05", "rtFingerPinky01", "rtFingerPinky02", 
			--	"rtFingerPinky03", "rtmetaCarpal02", "rtFingerIndex01", "rtFingerIndex02", "rtFingerIndex03", 
			--	"rtmetaCarpal04", "rtFingerRing01", "rtFingerRing02", "rtFingerRing03")
		
			pickbutton	pickBone	"[Click Me]"	align:#left		width:110	filter:bone_filt	message:"Select a Bone"	enabled:False

			button setupButton "Setup Mesh" width:220 toolTip: "Sets up the selected mesh."		enabled:False
			button spacer03 "" height:5 width:0
		)

		on pickBone picked obj do
		(
			pickBone.text = (obj.name as string)
			setupButton.enabled = True
		)

		on appendAttachmentFile pressed do
		(
			mergeMaxFile megaScene AttachmentScene
			
			appendAttachmentFile.enabled = False
			AttButtonsGroup = #(avatarType, pickBone, boneLabel)
				
			for a in AttButtonsGroup do a.enabled = True

			maleMesh = $Attachment_Male_Mesh...*
			femaleMesh = $Attachment_Female_Mesh...*
			hide femaleMesh
			hide maleMesh
			redrawViews()
		)
		
		on setupButton pressed do
		(
			if (classof $ == BoneGeometry) then
			(
				print("No mesh")
			)
			else
			(
				$.parent = undefined
				obj1 = "AttachmentRoot"
				obj1Path = execute ("$'" + obj1 + "'")
				obj2 = (pickBone.text as string)
				obj2Path = execute ("$'" + obj2 + "'")
				obj3Path = execute ("$" + obj1 + "/*[1]")

				obj1Path.rotation = obj2Path.rotation
				obj1Path.position = obj2Path.position

				obj3Path.name = "AttachmentNode" + "[" + pickBone.text as string + "]"

				$.parent = obj3Path
			)
		)

		on avatarType changed val do
		(
			if val == 1 do
			(
				maleMesh = $Attachment_Male_Mesh...*
				femaleMesh = $Attachment_Female_Mesh...*
				hide femaleMesh
				unhide maleMesh
				redrawViews()
			)
			if val == 2 do
			(
				femaleMesh = $Attachment_Female_Mesh...*
				maleMesh = $Attachment_Male_Mesh...*
				hide maleMesh
				unhide femaleMesh
				redrawViews()
			)
			if val == 3 do
			(
				maleMesh = $Attachment_Male_Mesh...*
				femaleMesh = $Attachment_Female_Mesh...*
				hide femaleMesh
				hide MaleMesh
				redrawViews()
			)
		)
	)

	rollout ClothingRollout "Clothing Tool" rolledup:true 
	(
		
		group ""
		(
			button appendClothingFile "Create Clothing" width:220 toolTip: "Create a new Clothing product"
			button spacer01 "" height:1 width:0
		)
		
		group ""
		(
			radiobuttons 	C_avatarType 	labels:#("Male      ","Female        ","Hide")	enabled:False
			button 			bindButton 		"Bind to Armature" 	width:220 	toolTip: "Weight the selected object to the armature."	enabled:False
			button 			paintWeights 	"Paint Weights" 	width:110 	toolTip: "Adjust the bone weighting."	across:2	align:#right	enabled:False
			button 			poseRig 		"Pose" 				width:110 	toolTip: "Pose the avatar."							align:#left	enabled:False
			button 			limitButton		"Limit Bones"	 	width:220 	toolTip: "Limit the bones."	enabled:False
			button 			spacer02 "" height:1 width:0
		)
		
		
		on appendClothingFile pressed do
		(
			mergeMaxFile megaScene ClothingScene

			ClothingRollout.appendClothingFile.enabled = False
			ClothingButtonsGroup = #(ClothingRollout.C_avatarType, ClothingRollout.bindButton, ClothingRollout.paintWeights, 
				ClothingRollout.poseRig, ClothingRollout.limitButton)

			for c in ClothingButtonsGroup do c.enabled = True
		)
		
		on bindButton pressed do
		(
			exportGroup = "CLOTHING_EXPORT"
			exportPath = execute ("$'"+ exportGroup + "'")
			
			allBones = $Clothing_Female03MasterRoot...*
			

			$.parent = exportPath
			clothingMesh = $
			
			max modify mode
			suspendEditing which:#modify
			clothingSkin = Skin()
			addModifier clothingMesh clothingSkin
			clothingSkin.name = "ClothingSkin"
			resumeEditing which:#modify

			for each in allBones do
			(
				skinOps.addBone clothingMesh.modifiers[#ClothingSkin] each 1
			)

			clothingMesh.modifiers[#ClothingSkin].showNoEnvelopes = on
			skinOps.voxelWeighting clothingMesh.modifiers[#ClothingSkin] 1 0.2 4 0 off on
			--$.modifiers[#ClothingSkin].bone_Limit = 4
		)

		on C_avatarType changed val do
		(
			if val == 1 do
			(
				maleMesh = $Clothing_Male_Mesh...*
				femaleMesh = $Clothing_Female_Mesh...*
				hide femaleMesh
				unhide maleMesh
				redrawViews()
			)
			if val == 2 do
			(
				femaleMesh = $Clothing_Female_Mesh...*
				maleMesh = $Clothing_Male_Mesh...*
				hide maleMesh
				unhide femaleMesh
				redrawViews()
			)
			if val == 3 do
			(
				maleMesh = $Clothing_Male_Mesh...*
				femaleMesh = $Clothing_Female_Mesh...*
				hide femaleMesh
				hide MaleMesh
				redrawViews()
			)
		)

		on poseRig pressed do
		(
			actionMan.executeAction 0 "415"
			subobjectLevel = 0
			-- local disp = NitrousGraphicsManager.GetActiveViewportSetting()
			-- disp.ViewportViewSettingImpl.VisualStyleMode = #Wireframe
		)

		on paintWeights pressed do
		(
			max modify mode
			local disp = NitrousGraphicsManager.GetActiveViewportSetting()
			disp.ViewportViewSettingImpl.VisualStyleMode = #Shaded
			subObjectLevel = 1

			skinOps.paintWeightsButton $.modifiers[#ClothingSkin]
		)

		on LimitButton Pressed do
		(
			$.modifiers[#ClothingSkin].bone_Limit = 4
		)
		
	)
/*
	rollout AnimationRollout "Animation Tool" rolledup:true 
	(
		group ""
		(
			button appendAnimationFile "Append Animation File" width:220 toolTip: "Append Animation File"
			button spacer01 "" height:1 width:0
		)
		
		group""
		(
			checkbox 	makePose 	"Make Pose"			width:220	align:#center
			checkbox	exportAll	"Export All Bones"	width:220	align:#center
		)

		group ""
		(
			spinner animationLength "Animation Length    " width:220 align:#center range:[1, 1000, 50] type:#integer
			--editText animationName "" width:220 align:#center toolTip:"Name your animation" text:"Animation Name"
			button spacer02 "" height:1 width:0
		)
		
		group ""
		(
			label 		IKFK 	"IK / FK"
			checkbox 	rArmIK 	"Right Arm" 	align:#right		checked:true	across:2
			checkbox 	lArmIK 	"Left Arm" 		align:#left 		checked:true
			checkbox 	rLegIK 	"Right Leg " 	align:#right		checked:true	across:2
			checkbox 	lLegIK 	"Left Leg" 		align:#left 		checked:true
		)

		
		group ""
		(
			label RotIso "Rotation Isolation"
			checkbox lArmRotIso "Left Arm" across:2
			checkbox rArmRotIso "Right Arm"
			checkbox lLegRotIso "Left Leg" across:2
			checkbox rLegRotIso "Right Leg"
		)
		

		group ""
		(
			button	addKey		"Add Key"		width:110	align:#right	across:2
			button	autoKey		"Auto Key"		width:110	align:#left
			button	resetPose	"Reset Pose"	width:220
		)

		on appendAnimationFile pressed do
		(
			mergeMaxFile megaScene AnimationScene
			frameRate = 30
			SetTimeScale animationLength
			actionMan.executeAction 0 "310"
		)
		
		on animationLength changed val do
		(
			SetTimeScale animationLength
		)
		
		on lArmIK changed val do
		(
			COG = $'WGT_COG'
			COG.Custom_Attributes.FKIKArmL = 100*lArmIK.triState
			select $WGT_FKArmLeg_001...*
			select $WGT_Hand_L
			select $WGT_PoleVector_002
			clearSelection()
		)
		on rArmIK changed val do
		(
			COG = $'WGT_COG'
			COG.Custom_Attributes.FKIKArmR = 100*rArmIK.triState
			select $WGT_FKArmLeg_019...*
			select $WGT_Hand_R
			select $WGT_PoleVector_003
			clearSelection()
		)
		on lLegIK changed val do
		(
			COG = $'WGT_COG'
			COG.Custom_Attributes.FKIKLegL = 100*lLegIK.triState
			select $WGT_Foot_L...*
			select $WGT_FKArmLeg_004...*
			select $WGT_PoleVector_004
			clearSelection()
		)
		on rLegIK changed val do
		(
			COG = $'WGT_COG'
			COG.Custom_Attributes.FKIKLegR = 100*rLegIK.triState
			select $WGT_Foot_R...*
			select $WGT_FKArmLeg_015...*
			select $WGT_PoleVector_001
			clearSelection()
		)

		on autoKey pressed do
		(
			isAutoOn = not isAutoOn
			max tool animmode
			if isAutoOn do
			(
				set animate on
				AnimationRollout.autoKey.text = "*Auto Key*"
			)
			if not isAutoOn do
			(
				set animate off
				AnimationRollout.autoKey.text = "Auto Key"
			)
		)

		on addKey pressed do
		(
			with animate on
			(
				rotate $ (angleaxis 0 [0,0,1])
				move $ [0,0,0]
				scale $ [1,1,1]
			)
		)

		on resetPose pressed do
		(
			for wgt in objects do
			(
				if (matchPattern wgt.name pattern:"*WGT*") do
				(
					-- print (wgt.name)
					selectmore wgt
				)
			)
			-- macros.run "Animation Tools" "RotationToZero"
			macros.run "Animation Tools" "TransformToZero"
		)
	)

*/

	rollout ExportRollout "Export Tool" rolledup:false
	(
		
		button exportButton 	"Export" 					width:220 	toolTip: "Export the Room or Furniture"
		button openDir			"Open Export Directory"		width:220	tooltip: "Open the export directory"
		button helpButton 		"Help" 						width:220 	toolTip: "Visit the Help Page"
		
		button spacer			""							width:0		height:5
		button versionButton 	updateText 					width:150 	enabled: isUpdateAvailable toolTip: "Current Version Number"
		
		on openDir pressed do
		(
			ShellLaunch "explorer" maxFilePath
		)

		on exportButton pressed do
		(
			newFileName = substring maxFileName 1 (maxFileName.count - 4)
			if newFileName == "" then
			(
				messageBox "Please save your project before exporting"
			)
			else
			(
				
				for o in objects do
				(
					if(matchPattern o.name pattern:"ROOM_EXPORT") do --ROOM
					(
						exportSeats = 0
						for o in objects do
						(
							if(matchPattern o.name pattern:"*seat*") do
							(
								exportSeats += 1
							)
						)
						
						if exportSeats < 7 then
						(
							messageBox "Not enough seats! You need at least 6"
						)
						else
						(
							messageBox ("Room exported as " + newFileName + "!")
							newFilePath = sysInfo.currentdir
							completeFilePath = (newFilePath + "/" + newFileName)
						
						
							roomExportGRP = getNodeByName "ROOM_EXPORT"
							select roomExportGRP
							
							fbxExporterSetParam "ScaleFactor" 1
							fbxExporterSetParam "UpAxis" "Z"
							
							exportFile completeFilePath #noPrompt using:FBXEXP
						)
					)
					if(matchPattern o.name pattern:"FURNITURE_EXPORT") do --FURNITURE
					(
						messageBox ("Furniture exported as " + newFileName + "!")
						newFilePath = sysInfo.currentdir
						completeFilePath = (newFilePath + "/" + newFileName)
					
					
						roomExportGRP = getNodeByName "FURNITURE_EXPORT"
						select roomExportGRP
						
						fbxExporterSetParam "ScaleFactor" 1
						fbxExporterSetParam "UpAxis" "Z"
						
						exportFile completeFilePath #noPrompt using:FBXEXP
					)
					if(matchPattern o.name pattern:"ATTACHMENT_EXPORT") do --ATTACHMENT
					(
						messageBox ("Attachment exported as " + newFileName + "!")
						newFilePath = sysInfo.currentdir
						completeFilePath = (newFilePath + "/" + newFileName)
					
					
						attachmentExportGRP = $ATTACHMENT_EXPORT...*
						select attachmentExportGRP
						
						fbxExporterSetParam "ScaleFactor" 1
						fbxExporterSetParam "UpAxis" "Y"
						
						exportFile completeFilePath #noPrompt using:FBXEXP selectedOnly:true
					)
					if(matchPattern o.name pattern:"CLOTHING_EXPORT") do --CLOTHING
					(
						messageBox ("Clothing exported as " + newFileName + "!")
						newFilePath = sysInfo.currentdir
						completeFilePath = (newFilePath + "/" + newFileName)
					
					
						clothingExportGRP = $CLOTHING_EXPORT...*
						select clothingExportGRP
						
						fbxExporterSetParam "ScaleFactor" 1
						fbxExporterSetParam "UpAxis" "Y"
						fbxExporterSetParam "Animation" False
						fbxExporterSetParam "BakeAnimation" False
						fbxExporterSetParam "Skin" True
						fbxExporterSetParam "Shape" False
						
						exportFile completeFilePath #noPrompt using:FBXEXP selectedOnly:true
					)
					if(matchPattern o.name pattern:"Animation_EXPORT") do --ANIMATION
					(
						newFilePath = sysInfo.currentdir
						completeFilePath = (newFilePath + "/" + newFileName)
						/*
						Objs = for o in objects collect o
						set animate on
						for frame = 0 to 49 do
						(
							for o in Objs do
							(
								if(matchPattern (o.parent as string) pattern:"*Animation*") do
								(
									if(matchPattern (o.parent as string) pattern:"*Bone*") do
									(
										if(matchPattern (o as string) pattern:"*lfShoulder*") do
										(
											sliderTime = frame
											pos1 = quatToEuler2 o.rotation.controller.value
											sliderTime = frame+1
											pos2 = quatToEuler2 o.rotation.controller.value
											
											if pos1 != pos2 do
											(
												addNewKey o (sliderTime-1)
												addNewKey o (sliderTime)
												o.rotation.controller.Orientation_Constraint.controller.weight[1] = 50
												
											)
										)
									)
								)
							)
						)
						set animate off
						*/
						animationExportGRP = $Animation_Bone_EXPORT...*
						select animationExportGRP
						
						-- fbxExporterSetParam "ScaleFactor" 1
						-- fbxExporterSetParam "ASCII" False
						-- fbxExporterSetParam "UpAxis" "Z"
						-- fbxExporterSetParam "GeomAsBone" True
						fbxExporterSetParam "Removesinglekeys" True
						-- fbxExporterSetParam "Resampling" 1
						fbxExporterSetParam "Animation" True
						-- fbxExporterSetParam "BakeAnimation" False
						-- fbxExporterSetParam "Skin" False
						-- fbxExporterSetParam "Shape" False
						-- fbxExporterSetParam "UseSceneName" True
						
						exportFile completeFilePath #noPrompt using:FBXEXP selectedOnly:true
						messageBox ("Animation exported as " + newFileName + "!")
					)
				)
			)
		)
		
		on helpButton pressed do
		(
			ShellLaunch "https://create.imvu.com/articles/studio/toolkit-3dsmax/" ""
		)

		on versionButton pressed do
		(
			ShellLaunch "https://create.imvu.com/articles/studio/toolkit/" ""
		)
		
	)

	IMVUFloater = newrolloutfloater "IMVU Studio Toolkit" windowWidth windowHeight
	addrollout VisualsRollout IMVUFloater
	addrollout RoomRollout IMVUFloater
	addrollout FurnitureRollout IMVUFloater
	addrollout AttachmentRollout IMVUFloater
	addrollout ClothingRollout IMVUFloater
	-- addrollout AnimationRollout IMVUFloater
	addrollout ExportRollout IMVUFloater
	

	for o in objects do
	(
		RoomButtonsGroup = #(RoomRollout.XSlider, RoomRollout.YSlider, RoomRollout.ZSlider, RoomRollout.furnitureType, RoomRollout.addFurnitureButton, 
				RoomRollout.addIdleButton, RoomRollout.addStandingButton, RoomRollout.addSittingButton,
				RoomRollout.customText, RoomRollout.addCustomButton, RoomRollout.createRoomST, RoomRollout.linkRoomST, RoomRollout.unlinkRoomST,
				RoomRollout.addCatcher, RoomRollout.addHandle, RoomRollout.removeCatcher, RoomRollout.removeHandle,
				RoomRollout.roomN, RoomRollout.roomW, RoomRollout.roomE, RoomRollout.rooms, RoomRollout.furnDist)

		FurnButtonsGroup = #(FurnitureRollout.F_addIdleButton, FurnitureRollout.F_addStandingButton,
				FurnitureRollout.F_addSittingButton, FurnitureRollout.F_customText, FurnitureRollout.F_addCustomButton,
				FurnitureRollout.F_addCatcher, FurnitureRollout.F_addHandle, FurnitureRollout.F_removeCatcher, FurnitureRollout.F_removeHandle,
				FurnitureRollout.F_createRoomST, FurnitureRollout.F_linkRoomST, FurnitureRollout.F_unlinkRoomST)

		ClothingRollout.appendClothingFile.enabled = False
			ClothingButtonsGroup = #(ClothingRollout.C_avatarType, ClothingRollout.bindButton, ClothingRollout.paintWeights, 
				ClothingRollout.poseRig, ClothingRollout.limitButton)

		AttButtonsGroup = #(AttachmentRollout.avatarType, AttachmentRollout.pickBone)

		if o.name == "ROOM_EXPORT" do
		(
			RoomRollout.createRoomButton.enabled = False
			FurnitureRollout.createFurnitureButton.enabled = False
			AttachmentRollout.appendAttachmentFile.enabled = False
			ClothingRollout.appendClothingFile.enabled = False
			
			for a in AttButtonsGroup do a.enabled = False

			for f in FurnButtonsGroup do f.enabled = False
			
			for c in ClothingButtonsGroup do c.enabled = False
				
			for r in RoomButtonsGroup do r.enabled = True
		)
		
		if o.name == "Room_Ref" do
		(
			RoomRollout.XSlider.value = ((o.scale.x * 100) / 2.5)
			RoomRollout.YSlider.value = ((o.scale.y * 100) / 2.5)
			RoomRollout.ZSlider.value = ((o.scale.z * 100) / 2.5)
			setScales 4000 4000 4000
		)
		
		if o.name == "FURNITURE_EXPORT" do
		(
			RoomRollout.createRoomButton.enabled = False
			FurnitureRollout.createFurnitureButton.enabled = False
			AttachmentRollout.appendAttachmentFile.enabled = False
			ClothingRollout.appendClothingFile.enabled = False
				
			for a in AttButtonsGroup do a.enabled = False

			for f in FurnButtonsGroup do f.enabled = True
			
			for c in ClothingButtonsGroup do c.enabled = False
				
			for r in RoomButtonsGroup do r.enabled = False
		)
		
		if o.name == "ATTACHMENT_EXPORT" do
		(
			RoomRollout.createRoomButton.enabled = False
			FurnitureRollout.createFurnitureButton.enabled = False
			AttachmentRollout.appendAttachmentFile.enabled = False
			ClothingRollout.appendClothingFile.enabled = False
				
			for a in AttButtonsGroup do a.enabled = True

			for f in FurnButtonsGroup do f.enabled = False
			
			for c in ClothingButtonsGroup do c.enabled = False
				
			for r in RoomButtonsGroup do r.enabled = False

		)

		if o.name == "CLOTHING_EXPORT" do
		(
			RoomRollout.createRoomButton.enabled = False
			FurnitureRollout.createFurnitureButton.enabled = False
			AttachmentRollout.appendAttachmentFile.enabled = False
			ClothingRollout.appendClothingFile.enabled = False

			for a in AttButtonsGroup do a.enabled = False

			for f in FurnButtonsGroup do f.enabled = False
			
			for c in ClothingButtonsGroup do c.enabled = True
				
			for r in RoomButtonsGroup do r.enabled = False
		)
		
		-- if o.name == "Animation_Scene" do
		-- (
		-- 	AnimationRollout.appendAnimationFile.enabled = False
		-- 	AnimButtonsGroup = #(AnimationRollout.animationLength, AnimationRollout.lArmIK, AnimationRollout.rArmIK, 
		-- 		AnimationRollout.lLegIK, AnimationRollout.rLegIK, AnimationRollout.makePose, AnimationRollout.exportAll,
		-- 		AnimationRollout.addKey, AnimationRollout.autoKey, AnimationRollout.resetPose)

		-- 	for an in AnimButtonsGroup do an.enabled = True
		-- )
	)
)
